#!/bin/sh
# This script produces a report of synspam
# Use it if you have a postfix and spamassassin setup


export LC_ALL=C
PATH=/bin:/sbin:/usr/bin:/usr/sbin:/usr/local/bin:/usr/local/sbin
REPORT="/tmp/synspam-report"
SASTATS="/usr/local/bin/sa-stats-ng.pl"
DAY=$(date --date '1 day ago' +"%b %d" | sed -e 's/0\([1-9]\)/\ \1/')
DRYRUN="true"
LOGS="/tmp/mail_logs"


function synspam_dry_run ()
{
echo -ne "\n\nSMTP connections blocked by synspam in dry run" >> $REPORT
echo -ne "which were not by spamassassin\n\n" >> $REPORT

> $LOGS

cat /var/log/mail.log.? /var/log/mail.log > $LOGS

grep "$DAY.*synspam\:.*reject" /var/log/daemon.log /var/log/daemon.log.? |while read line;do
#  IP=$(echo $line |cut -d ' ' -f7 | cut -d ':' -f1)
  IP=$(echo $line |egrep -o "[0-9]*\.[0-9]*.[0-9]*.[0-9]")
  REVERSE=$(host $IP)
  MATCH=$(grep -A 10 $IP $LOGS | egrep 'NOQUEUE|result')
  POS=$(echo $MATCH | egrep -v 'NOQUEUE|result\:\ Y')

  if [ -n "$POS" ]; then
          echo -ne "IP ADDRESS: $IP\n" >> $REPORT
          echo -ne "REVERSE: $REVERSE\n" >> $REPORT
          echo -n "$MATCH" >> $REPORT
          echo -ne "\n====================\n" >> $REPORT
  fi
done
}


as_top_ten ()
{
echo begin > /tmp/list.txt
grep "$DAY.*synspam\:.*reject.*" /var/log/daemon.log /var/log/daemon.log.? |egrep -o "[0-9]*\.[0-9]*.[0-9]*.[0-9]" >> /tmp/list.txt
echo end >> /tmp/list.txt

echo -ne "\n\nTop 10 spam sending AS\n\n" >> $REPORT

nc whois.cymru.com 43 < /tmp/list.txt |grep -v cymru | cut -d'|' -f 3 | sort -b | uniq -c | sort -r |head -10  >> $REPORT
}


# MAIN 

> $REPORT

echo -ne "====== synspam statistics ====== \n\n" >> $REPORT

as_top_ten

if [ -n "$DRYRUN" ]; then 
	synspam_dry_run
fi

if [ -f "$SASTATS" ]; then

	echo -ne "\n\n====== spamassassin statistics ===== \n\n" >> $REPORT

	if [ ! -d /tmp/sa ]; then
		mkdir /tmp/sa 
	fi

	grep "$DAY" /var/log/mail.log /var/log/mail.log.? > /tmp/sa/mail.log
	$SASTATS -l /tmp/sa -f mail.log >> $REPORT
fi

mail -s "`hostname -f` antispam report for $(date +%F --date '1 day ago')" root < $REPORT
